<?php
/*
 *  sif - simple image folder
 * 
 *   USAGE:
 *   -----------
 *    Just put this file into a web-folder, which is handled by a PHP-enabled 
 *    web server. (E.g. apache+mod-php).
 * 
 *    Let's say you have a directory /var/www/mysite, which is accessible under
 *    www.example.com.
 *    Now create a folder /var/www/mysite/pictures and put some pictures in it.
 *    Than also copy this file (index.php) into the same directory. If you now
 *    navigate with your browser to www.example.com/pictures this script gets 
 *    called and should generate a grid with downsized (thumb-nails) previews
 *    of your pictures. The first access will take some time - depending on
 *    your server and size of the pictures.
 *    But if the user-account, under which the script is executed can create 
 *    a new folder, the script stores the thumb-nails and can deliver them next
 *    time faster. 
 * 
 *     Daniel weigl, 2008
 * 
 * 
 *   LEGAL:
 *   ---------
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


// ---- CONFIGURATION ----
// Configuration - here are some options which change the outlook
// and/or behaviour of sif. 


// If this folder can be created (the user running the webserver needs write-privileges),
// the generated thumbnails will be cached into it. 
// This will speed up site-generating time extremely.
$cfg['thumb_dir']='./tmp';

// Grid layout
$cfg['cols']=5;
$cfg['rows']=4;	// -1 means no row-limit (no pages)

// Caption - if empty, the current foldername is used as caption
$cfg['caption']="";

// Show also subfolders
// 0...no
// 1...yes - just link to them
// 2...yes - and also handle them with sif
$cfg['link_folders']=2;

// Size of thumbnails (pixels)
$cfg['thumb_width']=120;
$cfg['thumb_height']=120;

// Border around thumbnails - in pixels
$cfg['thumb_border']=5;

// jpg-quality of the thumbs (0...worst, 100...best)
$cfg['thumb_quality']=80;

// size of preview - "-1" for original size. (pixles)
$cfg['preview_width']=640;
$cfg['preview_height']=480;

// Which function should be used to generate the thumbnails
// 0...resize (fast, bad quality); 1...resample (slow, better quality)
$cfg['thumb_resample']=1;

// File-filter
$cfg['filter']=array("jpg","jpeg","gif","png");




//--------------------------------------------------
// All following options are more advanced - only change them, if you know what you are doing :)
$cfg['version']='0.2';
// change this to e.g. "32M" if you get "Allowed memory size of 16777216 bytes exhausted "
$cfg['memory_limit']="";



// - Start of sif - here we go....
$mode=$_GET['sif'];
$sif=new sif($cfg);
$sif->Render($mode);
// - End of sif - thats all folks!


// -------- Functions & classes
class sif{
	public $cfg;
	
	private $mode, $thumber, $base_folder, $dirs, $files,$dirs_and_files, $sites;
	
	function __construct($cfg){
		if ($this->cfg['memory_limit']!="") ini_set('memory_limit', $this->cfg['memory_limit']);
		$this->cfg=$cfg;
		$this->base_folder=$this->get_base_folder();
		$this->files=$this->getFileListing($this->base_folder,false);
		$this->dirs=$this->getFileListing($this->base_folder,true,false);
		
		$this->dirs_and_files=array_merge($this->dirs,$this->files);
	}
	
	function Render($mode){
		$this->mode=$mode;
		if (($mode=='')||($mode=='show')){
			$this->RenderSIF();
		}elseif ($mode=='prev'){
			$this->RenderPreview();
		}elseif ($mode=='pic'){
			ReturnPic($_GET['p']);
		}elseif ($mode=='thumb'){
			$this->RenderThumb($_GET['p']);
		}
	}
	
	function RenderSIF(){

		
		$this->thumber=$this->GetThumber();
		
		$str=$this->RenderTitle();
		$str.=$this->RenderFolder();
		$str.=$this->RenderTable($this->dirs_and_files);
		
		$ptit=$this->PageTitle();
		
		echo $this->html($str,$ptit);
	}
	
	function RenderPreview(){
		$str=$this->RenderTitle();
		$str.=$this->RenderFolder();
		$str.=$this->RenderPreviewPic();
		echo $this->html($str,$this->PageTitle());
	}
	
	function RenderPreviewPic(){

		$this->thumber=$this->GetThumber($this->cfg['preview_width'],$this->cfg['preview_height']);
		$files=&$this->dirs_and_files;
		$pic=$_GET['p'];
		
		if (array_search($pic,$files)===false) return "";
		
		$img=$this->thumber->get_html($pic);
		
		
		$this->thumber=$this->GetThumber();  // get thumber for small thumbnails
		
		$idx=array_search($pic,$this->files);
	
		$cnt=count($this->files);
		
		$arg=$_GET;
		$arg['p']=$this->files[max(array($idx-1,0))];
		$prev="?".http_build_query($arg);
		
		$arg['p']=$this->files[min(array($idx+1,$cnt-1))];
		$nx="?".http_build_query($arg);
	
		$s.="<div align=center><a href=\"$prev\"><img border=0 src=\"?sif=pic&p=5\">&nbsp;prev</a>&nbsp;".($idx+1)."/$cnt&nbsp;<a href=\"$nx\">next&nbsp;<img border=0 src=\"?sif=pic&p=6\"></a></div>";
		
		$plink=my_htmlentities($pic);
		$s.="<div align=center><table class=\"pic_brd\"><tr><td>
		<div align=center>
		<a href=\"$plink\">$img</a>
		</div>
		</td></tr></table></div>";
		
		$start=max(array($idx-2,0));
		$cols=5;
		$s.="<div align=center><table class=\"pic_grid\" border=0>\n";
		$s.=$this->RenderPictureRow($this->files,$start,$cols);
		$s.="\n</table></div>";
		
		return $s;
	}
	
	function RenderTitle(){

		$link=basename(realpath($this->base_folder));
		$akfolder=nice_chars($link);
		$fl=urlencode($this->relative_folder($this->base_folder));
		if ($this->cfg['caption']!=""){
			$caption="<a href=\"?folder=$fl\">".$this->cfg['caption']."</a>";
		}else{
			$caption="Pictures from &#187;<a href=\"?folder=$fl\">$akfolder</a>&#171;";
		}
		$s="<p class=title>$caption</p>";
		return $s;
	}
	
	function PageTitle(){
		$title=nice_chars(basename(realpath($this->base_folder)));
		if ($this->sites[0]>1){
			$title.=" (Page ".$this->sites[1]." of ".$this->sites[0].")";
		}
		return $title;
	}
	
	function RenderFolder(){
		$rel=$this->base_folder;
		if ($rel==".") return "";
		
		$rel=explode(DIRECTORY_SEPARATOR,$rel);
		
		$parms=$_GET;
		unset($parms['site']);
		unset($parms['sif']);
		
		
		$pth=array();
		array_unshift($rel,basename(realpath("."))); // add the root folder also to the bread-crumps
		$notfirst=0;
		foreach ($rel as $dir) {
			if ($notfirst==1) $folder.=$dir.DIRECTORY_SEPARATOR;
			$notfirst=1;
			$parms['folder']=$folder;
			$flink="?".http_build_query($parms);
			$dir=nice_chars($dir);
			$pth[]="<a href=\"$flink\">$dir</a>";
		}
		$pths=implode("&nbsp;~&nbsp;",$pth);
		
		$s="<p class=\"folder\"><b>Folder</b>: $pths";
		
		$s.="</p>";
		return $s;
	}
	
	function GetThumber($w=NULL,$h=NULL){
	
		if (!isset($w)) $w=$this->cfg['thumb_width'];
		if (!isset($h)) $h=$this->cfg['thumb_height'];
		
		$th=new ThumbGen_cache($w,$h);
		if ($th->works()) return $th;
		
		$th=new ThumbGen_ondemand($w,$h);
		if ($th->works()) return $th;
		
		$th=new ThumbGen_onclient($w,$h);
		return $th;
	}
	
	function get_base_folder(){
		if ($this->cfg['link_folders']==2){
			$folder=$_GET['folder'];
		}
		
		$basefolder=realpath(".".DIRECTORY_SEPARATOR.$folder);
		$basefolder=$this->relative_folder($basefolder);
		return $basefolder;
	}
	
	function relative_folder($path){
		$base=realpath(".");
		$path=realpath($path);
	
		$same_dir=substr($path,0,strlen($base));
		if ($base==$path){
			return ".";
		}
		
		// Ensure that the given path is relative to my base-path, otherwise return ./
		if ($same_dir!=$base){
			return ".".DIRECTORY_SEPARATOR;
		}else{
			return substr($path,strlen($base)+1);	
		}
		
	}
	
	function RenderImgPreviewHTML($filename){
		
		$nice_filename=nice_filename($filename);
		
		$pic=$this->thumber->get_html($filename);
		
		$arg=$_GET;
		$arg['sif']='prev';
		$arg['p']=$filename;
		$plink="?".http_build_query($arg);
		
		$nice_filename=my_htmlentities($nice_filename);
		$s="<table class=\"pic_brd\"><tr><td>
		<div align=center>
		<a href=\"$plink\">$pic</a><br>
		<div class=\"pic_txt\"><a href=\"$plink\">$nice_filename</a></div>
		</div>
		</td></tr></table>";
		return $s;
	}
	
	
	function RenderFolderPreviewHTML($folder){
		
		$nice_folder=nice_filename($folder);
	
		$nice_folder=my_htmlentities($nice_folder);
		
		if ($this->cfg['link_folders']==1){
			$flink=urlencode($folder);
		}else{
			$parms=$_GET;
			$parms['folder']=$folder;
			unset($parms['site']);
			$flink="?".http_build_query($parms);
		}
		
		$s="<a href=\"$flink\" border=0px><img border=0 src=\"?sif=pic&p=1\">
		<br>$nice_folder</a>";
		return $s;
	}
	

	
	function RenderTable(&$files){

		
		$cols=$this->cfg['cols'];
		$rows=$this->cfg['rows'];
		
		$cnt=count($files);
		if ($cnt==0) return "";
		if ($rows>0){
			$show_pics=$rows*$cols;
			$sites=ceil($cnt / ($show_pics));
			$sitebar="<tr><td colspan=$cols><div class=\"sitebar\">".$this->RenderSiteBar($sites)."</div></td></tr>";
			$aksite=$this->get_current_site($sites);
			$img_offset=($aksite-1)*$cols*$rows;
			$end_pic=$show_pics+$img_offset;
			$end_pic=min(array($end_pic,$cnt));
			
			// store current and max sites for later usage
			$this->sites=array($sites,$aksite);
		}else{
			$img_offset=0;
			$end_pic=$cnt;
			$this->sites=array(1,1);
			$rows=ceil($cnt/$cols);
		}
		
	
		$s="<table class=\"pic_grid\" border=0>\n$sitebar\n";
		
		$start=$img_offset;
		for ($r=1;$r<=$rows;$r++){
			if ($start>=$cnt) break;
			$s.=$this->RenderPictureRow($files,$start,$cols);
			$start+=$cols;
		}
		
		$s.="\n$sitebar\n</table>";
		
		return $s;
	}
	
	function RenderPictureRow($files,$start,$cols){
		$s="\n<tr>";
		$remain_cols=$cols;
		for($i=$start;$i<$start+$cols;$i++){
	
			$akf=$files[$i];
			if (!isset($akf)) break;
			$remain_cols--;
			if (is_dir($akf)){
				// a directory
				$cell=$this->RenderFolderPreviewHTML($akf);
			}else{
				// a picture
				$cell=$this->RenderImgPreviewHTML($akf);
			}
			$s.=" <td align=center>\n$cell\n</td>\n";
		}
		if ($remain_cols>0) $s.=str_repeat("<td></td>",$remain_cols);
		$s.="\n</tr>";
		return $s;
	}
	
	function get_current_site($max_sites){
		$aksite=$_GET['site'];
		$aksite=max(array(1,$aksite));
		$aksite=min(array($max_sites,$aksite));
		return $aksite;
	}
	
	function RenderSiteBar($sites){
		if ($this->cfg['rows']<=0) return "";
		if ($sites<=1) return "";
		$aksite=$this->get_current_site($sites);
		return $this->site_links_bar($aksite,$sites);
	}
	
	function getFileListing($folder,$subfolders=true,$include_files=true){
		$dirs=array();
		if ($this->cfg['link_folders']>0 && $subfolders){
			//$dirs=glob($folder."/*",GLOB_ONLYDIR);
			$dirs=my_glob($folder,"",true,false);
			sort($dirs);
			//Remove the temp-dir
			$dirs=array_diff($dirs,array($this->cfg['thumb_dir']));
		}
		
		$files=array();
		if ($include_files){
			//$files=glob($folder."/".$cfg['filter'],GLOB_BRACE);
			$files=my_glob($folder,$this->cfg['filter'],false);
			sort($files);
		}
		return array_merge($dirs,$files);
	}
	
	
	function RenderThumb($fn){
		$w=$this->cfg['thumb_width'];
		$h=$this->cfg['thumb_height'];
		if (isset($_GET['w'])) $w=intval($_GET['w']);
		if (isset($_GET['h'])) $h=intval($_GET['h']);
		
	
		if (array_search($fn,$this->files)!==false){
			$is=getimagesize($fn);
			//$mime=image_type_to_mime_type($is[2]);
			header('Content-type: image/jpg');
			$thumb=$this->ThumbResize($fn,$w,$h,$is);
			
			if ($thumb!==false){
				echo imagejpeg($thumb, null, $this->cfg['thumb_quality']);
			}else{
				ReturnPic(2);
			}
		}
	}
	
	function ThumbResize($fn,$w,$h,&$is=null){

		if (is_null($is)){
			$is=getimagesize($fn);
		}
		$ow=$is[0];
		$oh=$is[1];
		
		if ( $ow==0 || $oh==0 ){
			return false;
		}
		
		if ($w>$ow) $w=$ow;
		if ($h>$oh) $h=$oh;
		
		if ($ow>$oh){
			$fac=$w/$ow;
		}else{
			$fac=$h/$oh;
		}
		
		$nw=$ow*$fac;
		$nh=$oh*$fac;
		
		$image = imagecreatefromstring(file_get_contents($fn));
		//$dst_im=imagecreate($nw,$nh);
		$dst_im=imagecreatetruecolor($nw,$nh);
		
		// Get the global cfg-var if called static (i know... bad style)
		if (!isset($this->cfg)) {
			global $cfg;
		}else{
			$cfg=&$this->cfg;
		}
		
		if ($cfg['thumb_resample']==1){#
			imagecopyresampled($dst_im,$image,0,0,0,0,$nw,$nh,$ow,$oh);	
		}else{
			imagecopyresized($dst_im,$image,0,0,0,0,$nw,$nh,$ow,$oh);
		}
		return $dst_im;
	}
	
	function site_link($site,$text=null,$url_only=false){
		$site=intval($site);
		$arg=$_GET;
		$arg['site']=$site;
		if (is_null($text)) $text=$site;
		$args=http_build_query($arg);
		if ($url_only){
			$s="?$args";
		}else{
			$s="<a href=\"?$args\">$text</a>";
		}
		return $s;
	}
	
	function site_link_segment($start,$end,$current_site){
		$ret=array();
		for($s=$start;$s<=$end;$s++){
			if ($s==$current_site) {
				$b1="<b>";$b2="</b>";
			}else {
				$b1=$b2="";
			}
			$ret[]=$b1.$this->site_link($s).$b2;
		}
		return implode("&nbsp;",$ret);
	}
	
	function site_links_bar($current_site,$max_sites){
		if ($max_sites<10){
			$ret=$this->site_link_segment(1,$max_sites,$current_site);
		}else{
			$ret=$this->site_link_segment(1,3,$current_site)."&nbsp;";
			$ms=(max(array($current_site-1,4)));
			$me=min(array($current_site+1,$max_sites-3));
			if ($current_site>5) $ret.="...&nbsp;";
			$ret.=$this->site_link_segment($ms,$me,$current_site)."&nbsp;";
			if ($current_site<$max_sites-4) $ret.="...&nbsp;";
			$ret.=$this->site_link_segment($max_sites-2,$max_sites,$current_site);
		}
		
		$ret=$this->site_link($current_site-1,"<img border=0 src=\"?sif=pic&p=5\">&nbsp;prev ").$ret.$this->site_link($current_site+1," next&nbsp;<img border=0 src=\"?sif=pic&p=6\">");
		return $ret;
	}
	
	function generateCacheName($filename,$w,$h,$ext="jpg"){
		$filename=realpath($filename);
		$hash=md5($filename."-".filectime($filename))."_${w}_${h}.$ext";
		return $hash;
	}
	
	// --- Mostly html/css functions from now on

	function get_link_rels(){
		$s="<LINK REL=\"Home\" HREF=\"?\">\n";
		
		if ($_GET['sif']=='prev'){
			$files=&$this->files;
			$ak=array_search($_GET['p'],$files);
			$arg=$_GET;
			$arg['p']=$files[max(array($ak-1,0))];
			$prev="?".http_build_query($arg);
			
			$arg['p']=$files[min(array($ak+1,count($files)-1))];
			$nx="?".http_build_query($arg);
			
		}else{
			$ak=$this->sites[1];
			$max=$this->sites[0];
			if ($max==1) return $s;
			$prev=$this->site_link(max(array($ak-1,1)),"",true);
			$nx=$this->site_link(min(array($ak+1,$max)),"",true);
		}
		
		$s.="
			<LINK REL=\"Previous\" HREF=\"$prev\">
	    <LINK REL=\"Next\" HREF=\"$nx\">";
		
		return $s;
	}
	
	function js(){
		$s=<<<JS
		function fit_width(obj){
		 var my_w=Math.max(obj.width,50);
		 var tx_field=obj.parentNode.parentNode.childNodes[4];
		 if (!tx_field) return;
		 if (tx_field.style) tx_field.style.width=my_w+"px";
		}
JS;
		return $s;
	}

	function css(){

		$width=$this->cfg['thumb_width'];
		$border=$this->cfg['thumb_border'];
		$border_b=$border-2;
		$s=<<<CSS
	<style type="text/css">
body {
  font-family: Arial,Helvetica,sans-serif;
  background-color: #fdfdfd;
  margin-top: 0px; 
  padding-top: 0px; 
  top: 0px; 
  margin-bottom: 0px; 
  padding-bottom: 0px; 
  bottom: 0px;
  
}
.col {
  padding-right: 10px;
  padding-left: 10px;
  background-color: #e8f3fc;
  padding-bottom: 10px; 
}
.bleft {
  background-image: url(?sif=pic&p=3);
  background-repeat: repeat-y;
  background-position: right top;
}
.bright {
  background-image: url(?sif=pic&p=4);
  background-repeat: repeat-y;
}

.pic_brd {
  border: 0px solid white;
  padding-top: ${border}px;
  padding-bottom: ${border_b}px;
  padding-left: ${border}px;
  padding-right: ${border}px;

  margin-top: 10px;
 	margin-bottom: 0px;
 	margin-left: 5px;
 	margin-right: 5px;
  font-size: 1em;
  background-color: #FCFCFC;
}

.pic_grid {
 text-align: left; 
}

.pic_txt {
	width: ${width}px;
	margin-top: 5px;
	padding-top:3px;
	padding-bottom:3px;
	border: 1px solid #F4F4F4;
	background-color: #FFFFFF;
	font-size: 0.85em;
	overflow: auto;
}

.title{
  font: italic 1.8em georgia; 
}

.folder{
  font: italic 1em georgia; 
  padding-left: 30px;
}

.pic {
  border: 1px solid #a2aab0;
}

.footer{
	font-size: 0.8em;
	color: #A2AAB0;
}

.sitebar{
	font-size: 0.86em;
	text-align: right;
}

*.footer hr{
	color: #A2AAB0;
	background-color: #A2AAB0;
	height: 1px;
	border: 0px;
	width: 50%;
}

 a:link{
    text-decoration: none;
    color: #000000;
}

 a:visited{
    text-decoration: none;
    color: #000000;
}

 a:hover{
    text-decoration: none;
    color: #003ADA;
}

*.footer a:link{
    text-decoration: none;
    color: #A2AAB0;
}

*.footer a:visited{
    text-decoration: none;
    color: #A2AAB0;
}

*.footer a:hover{
    text-decoration: none;
    color: #003ADA;
}
  </style>
CSS;
		return $s;
	}

	function html($body,$page_title="sif"){

		$css=$this->css();
		$page_title=my_htmlentities($page_title);
		$link_rel=$this->get_link_rels();
		$version=$this->cfg['version'];
		$js=$this->js();
		$s=<<<HTML
<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
<head>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
<title>$page_title</title>
$css
$link_rel
<script language="JavaScript">
$js
</script>
</head>
<body>
<table align="center" border="0" cellpadding="0"
 cellspacing="0">
  <tbody>
    <tr>
      <td class="bleft" style="width: 5px;"></td>
      <td class="col">
      $body
      <br><br><br>
			<div class="footer" align="center">
			<hr >
			This site was generated by simple image folder $version <br>
			<a href="http://sourceforge.net/projects/simpleimage">Get sif</a>
			
			</div> 
			<br>
			</td>
      <td class="bright" width="5"></td>
    </tr>
  </tbody>
</table>

</body>
</html>
HTML;
		return $s;
	}
	
}

class ThumbGen{
	protected $width;
	protected $height;
	
	function __construct($w,$h){
		$this->height=$h;
		$this->width=$w;
	}
	
	protected function have_gd(){
		return (GD_VERSION!="");
	}
	
	protected function img_html($src,$additional_params=""){
		return "<img border=0 src=\"$src\" $additional_params onload=\"fit_width(this)\">";
	}
}

class ThumbGen_cache extends ThumbGen {
	
	public function works(){
		global $cfg;
		
		if (!$this->have_gd()) return false;
		if (!file_exists($cfg['thumb_dir'])){
			@mkdir(".".DIRECTORY_SEPARATOR.$cfg['thumb_dir']);
		}
		if (!file_exists($cfg['thumb_dir'])){
			return false;
		}
		
		/*
		if (!touch($cfg['thumb_dir']."/sif_testfile")){
			return false;
		}else{
			unlink($cfg['thumb_dir']."/sif_testfile");
		}
		*/
		
		return true;
	}
	
	private function cache_pic($filename,$to){
		global $cfg;
		$thumb=sif::ThumbResize($filename,$this->width,$this->height);
		if ($thumb!==false){
			return imagejpeg($thumb,$to,$cfg['thumb_quality']);			
		}else{
			return false;
		}
	}

	private function have_thumb($big_fn){
		global $cfg;
		$fn=$cfg['thumb_dir'].DIRECTORY_SEPARATOR.sif::generateCacheName($big_fn,$this->width,$this->height);
		
		if ( !file_exists($fn) ) {
			$okay=$this->cache_pic($big_fn,$fn);
			if ($okay) return $fn; else return false;
		}
		return $fn;
	}
	
	public function get_html($filename){
		
		$fn=$this->have_thumb($filename);
	
		// Failback to on-demand resizing if unable to store thumbnail 
		if (!($fn===false)){
			$s=$this->img_html($fn);
		}else{
			$s=ThumbGen_ondemand::get_html($filename);
		}
		return $s;
	}
}

class ThumbGen_ondemand extends ThumbGen {
	public function works(){
		return $this->have_gd();
	}
	
	public function get_html($filename){
		$arg=$_GET;
		$arg['sif']='thumb';
		$arg['p']=$filename;
		$arg['w']=$this->width;
		$arg['h']=$this->height;
		$args=http_build_query($arg);
		$s=$this->img_html("?$args");;
		return $s;
	}
}

class ThumbGen_onclient extends ThumbGen {
	public function works(){
		return true;
	}
	
	public function get_html($filename){
		$s=$this->img_html($filename," width=".$this->width." height=".$this->height." ");
		return $s;
	}
}




// the following function can be updated with the img_code.php script (from the svn-repo)
//	(on the commandline in the same folder as index.php, execute: 
//     $ php img_code.php
//	)
// ***PIC-start

function ReturnPic($picture){
	$picture=intval($picture);
	
	if ($picture==1) {
		$b64="R0lGODlhNwA4AOf/AAAy3gA9sRA+vBY62AhDsgJB7B884iA/xiFCqwtJvhVHtiU+7SZB0CdA1wtI3xZF4QBK9BtJshhE8QpOvABR
		      4xNRuCRPqiZK5yBPzSpK8ABW9yxO1RxQ9DJN3ABc7RRcwy1XrSdV4yhavSlV8jhU3QVh+yFc+C9fuyZjxTRc4ydg5Rdn0kNgqzpi
		      si1i7x1m+S9i9wlt/zhmvCJtzkhmqy9sxyhq/UxntCJy0ztq6kBvvFNspThu5jJv/BN4/1Jp3yZ0/yV52kNz4Tt0+19ypx5+/zZ8
		      2DJ6/112qVp2vV93sj5/1U98yUp82lN9wj2A5215qiyE/zyA/z+D2jiH3EOF1EuA+lGB4GaBr2Z951iE0DeK/0WH/z6O5XeDtUSR
		      4D+Q/zKU/0yN/zyV6VmP2m+MzX+Ltk6U8WqM702X516T3kiX/1OU/zif8mmU2FmZ40ae7GCY24uSs1ia/06d/1We7k2j8UGn9HWb
		      3IiYxGif5Fij7F6g/36Z8VWk/2Kj53Gg4Jibt1ir82Sn/1ur/5af2Wuq7XCq/Yen2mqu65iky2Sw82iu/3at5Wuw/4Kq+qSmv4uq
		      53ay/nK182q49I+v4m62/5+r34C17m+6/oiz/Im16Xm4/puz2KeywK+wyXK9/4W67Xy89Hy7/5i36nTC/o+7/5a763/B/6i45Yi/
		      /5u+45S/9JDB8ri72Lu70ovG/4TK/7DB5I3K+8O/0ZXJ/ajI5aHJ8JfL/4fQ/5DO/6jJ85vN9LXG8K3J7cbE1ZfP/LjI4rLK6cbG
		      3ZHT/srG2LrJ6qbN/6HP/pzU/8fL26nS+63S9bbT+r7R+KXY/57b/83R4MfW7rXa/a/d/qjf/9fX4sXd/L/f/bni/sbg9rPl/9bd
		      7NDf/97e6dPj/Lzq/sXp/9/k5szo/7/t/+Xm8Njp/sfw/9Xt/+Ls+9Dx/+rr9tbz/tD1/t7y//Du8sr4//Xz9+D5/u71/tr7/tT9
		      /+n5/93+//n3++78/en//v77//L///n///7//P///yH5BAEKAP8ALAAAAAA3ADgAAAj+AP8JHEiwoMGDCBMqXMiwocOHECNKnEix
		      osWLGBkiYhIsY0ZjNWrgOeHRojQtKEhNOzbBU7AaE5iUdJgO0IdK00YR8rNCBopKxkjOXFhpBqBruiwx8vPkA55vkYQoGJrQVw0y
		      16iVymTpzIolzjQ1aSKVqkFpS5bsCherVClGQWb4arbFg5ArPBKYHWhOjZFT7IjFigWqywxS3/gUOfKCx5UcCcxYvJXIkK+C6Rrh
		      2OQOGrFcseoEUZPOVJQii22oaKICgxKKt+BMGjctTRxxAk8tAYQunGdimaZQ+XZszpbTi4GE4JECAwiJzOxMQudPH712hqjEoaJH
		      m7pq0KD+zRqj1h2jNWCOoz7iw0WIFBsIRFwkaJw/fvru0btOrdY4edtsU00ye0yByTyq+EEHeuot5kMMDpDQQQAQ2QFMP/3gl99+
		      7bRzDjjgBDhJF3+YMw0jOy2YHnLs2fAABR0I8JAg0/iToYb6XechiNXAkcY146Ci1CAKMsiiDyZc8ECMrzXUhj/38YMjhzuCs8ge
		      /SSDyihKpWjkejaMMIIBDhzQgkNt5BPllDp+CM4Y7mQDy5ZDFgkGF8gBYYIGHBgwAAPyNZQINmvqs2Gb4DzzhT+6zMklI0SqGIUU
		      qNlgAgcZGABAAxQ2FMozhR7a4YezGJIPLo4OyceCx1EaQwn+l0JQAAADCIAFQ7zcoaaUbI4Kzh7LoNMoKpxIAikfa4ShXg8vmDAC
		      BxAsoKmZCzFzhzs28mpojh2CM8kY8/Q3Z7FLqXgcEDbAsCcHEiywAAAMIMBQmtlKuS2H1MBhyDzhAFPLnJY44iUYURzRQ7p7ZiBB
		      AQv88EkEDIUCTL1TtjMJHMzkk80z/tKJYpFcHAHEwSU8K+sIhbzyShI0LKSNHRRvm4wdotjDjoDJDPsokWuIUfDIlpqchSu00PKK
		      IvIutAeh2rZDiR3j6HOOgM/knOpSc7CxhRQGx6CuBjBcUswvvxj9CsQLrTLJmtlYyA89H1Jt9ZaSDMLHHHdKMcT+weqiMYwyyoxd
		      9MpnLjSGPTdSMk0/9cATdzVVD6tqz1zoffAQqUQTDeBjlw0JEZ0q1EgrUVKCjj6NP141qqMYO8iCYoQ8ch/RWGON5oEX84ocLASA
		      BEPcjHGjKNOgvqPctdCNNRs+H2FFL97YfvvmynwCRQQWPPTHMvcBMws/qYcIOTCO1n231lI8wo037Nuu+S+B9L4DRLakcV82i4B/
		      PDXPDMuJI3bLGhucUY4Cts92r/AC9ibyhcXJow32qMf+rKYKVc3BFOlYxzoMyL5oQOIGAZjfRFbxhuosAhv0cFyIqjYLWOzsEN2I
		      xzvesUEDKsMMEXiORahgDn4AoxX++tjR+JLXOkYcYx34iIcMNVgOb3xCBwggQkb04I9wpKEfqiPfllRhjn7sYx9KnOE6opGHHJZk
		      OPyAAzZUN6zFQemLSYxHOVYWxaGUEBOtoIf4coYMe0AJSl4EozcUUQEdDmUJzcBGGvixR/v88Y9gjIYTIvA7s3RiCv3owjhClA1+
		      PPKR7+iECAy5lyqcghWYYAc7PvlIbpShApXci0CCMYNrUIGVkJTFCQonS4LEQQ1faAYuy6GFD0ChlweZQRyo+Mh9yAIFvERmQTqB
		      giDM44/rcAMKbiVNhDBhBpuAkjBmoINuLmQGRpiHG2rATXMmBBEzmIET3EnPetrznvgFzOdDAgIAOw==";
		header('Content-type: image/gif');
	}
	
	if ($picture==2) {
		$b64="iVBORw0KGgoAAAANSUhEUgAAADwAAAArCAIAAACrTXwGAAAACXBIWXMAAABIAAAASABGyWs+AAAR40lEQVRYw31ZWbMdV3X+1tq7
		      u885fcZ77qRZsoVtSUaWjRkkQxnCYGJIwBTkIaTykoL8gFQlbxkgJI9Jkar4gYoDJiYhIVNhbKYCbFkehDXYlmxJtqzRGu88nHO6
		      e++18rD7nHtlqrLr1q079Olee+1vfetbX9Ppk1cAEJGI6HCxAKqqCgBA+EHEq4oIVLUofJ45UVHP3kPEi6gXFa/iVeFATnz4IKsS
		      ICqGiAkKElFHBFGnypCECN5rkTtVJbLeexHx3qmqiBMRAMwMWGMMkVoiCpExc4gYqgSFkq6LO2wsfFNVZjKG4VUYDFIlIjABDCgU
		      pESkgITPkSoTKyAKAAoQlAhWBOI9APEQQZ67ohgUReG9d4Xz4olCgGSMieMkssqG1oIOGQ3PYiIQRol/V9BERMTWkogwh20wUMao
		      qgQCMSkEKhLyoEQgUhURAZRFyTnxXl2RZ1ne6w2Wl1aWl1eKoigK51xBxMYYa60xNi5XVqtVKtXKLUEbNgotQ1iHDSIKT10XNJjZ
		      GOOcB0CkzAAIgCqpkpZXwhgWKe8k4kSUYArnndN+P1tcWJyZmZmfX1hcXB70B0XhmA0RARpFURxX4jiJoqRSkSTWInFEZIxdC1pV
		      RSU8C4TRVowxROS9iCggZdBQYrLWeu8BJVIlsGFAVCngHmBmImJARcR7UYUKZXmxsrJy/dqNy5ev3Lx5s9dfzbJMhYwxQIl7YioK
		      mxd5TerMtsgdgY1FnrvBILchoyF0Zi6RgHevkLZwITOrIfHKzFFkRUTVAwGuZZpZWYmhcF4AEhEoi+PZ2dm3z719+fKlmZmZ1dWe
		      qrfWELNCRS2RB1ihEAGswufWREVkrQVFIioiANlRotfjhG4Jd3QUBFUCiBis4j0TW2u8R8ApAFWwErFVZVeIqDBR4STLsqXF/pnT
		      506ffmNuftb7nEij2DIbIog4hYoIERMxADZQQJRFnKonUmZShYgQkSU2a/CQspICIrUMlwAQG2by4tV5EVFWsiQq8DGTWpsTORGF
		      MhSqIkrM5B25AstLg7fPXnr1xMtXrp3LsyKKYhtFzBZKrihLPYoi751IYSxbYwzFKsyUkCaMxFDERIrcRnGlaqyoDLNLzKSqZT2F
		      aJVEtcw8E4HJaMlqBBIiVVUKCIMPRAnvSURFUBTu+vXZIy8fPXP67PLqjGKQJNUoSpgsYIyxNoriqBJFkcJ575aWFvM8Y4KSGhMR
		      GMqRTeIoISJjqFarxHFkSxgrAqEFeir5tMSpkNIQKWwMAIROBICMChyTDClYvRCzUZV+f+XM6TePHDl64cIF5wpmW0kmkiRhY6uV
		      WqPRbLc7tVq1UqkS8erq8tzcjHda5HMqxJExxhq2SRLHiTUWzKjX62maGmOsJV6PYg18SyNwh4MIiGGCBp4PcRMRkTILWOGVmcUr
		      EzIn/V5+8sQbzx58dn5+RlXjxKS1Vhy1mo36xMTU2Fh3bKyTpqmIiKhzRRxFvV4/z50IkiSxNiEYa6J6vVGtVo3hSjVqNBpRFBGR
		      BY3gARBBFBDDo+IDSv5SIh5etcaSzCRQSwbG5pkQwTnXWx0cP378ueeem5+fB3G1Wm23m7VKa2Jiy4bp6cnJiWazwYZVZXW1P+hn
		      gA6ybHb25tLSYpxExrDhiCiqVmuNRsNajmLTbjdrtaoxhpktY61RI3QQIkHZWojAJWYApdGJlAzMDOdULRG5wjNFeZb1VouXXz72
		      q1/9cjAYENlOe6zV6tTrjW3btk5NTXY6nXqjJuK8L7x3ogNiDHr9y5fPXbx0TtVVKmkSx1Cq1WpjY+NxHFlLY51Wo5GayIanW8aI
		      IhSqxExAAVlPeWU3l1JbqErYEzODDcNCAdUs83kmx46eeP7Q86qSJJWJ8elmszPWnty+/fbOWK3ZTiqVuCgyUAF2Rb6qmnlPZ868
		      fvrMG3k+aLWalao1xLFNu2MT9XqdgLFOp9Wqm4gUEBFmtsIMUgLECzOrQryAlAmqRGQACZ3cEVQFEJRkGAShV+NVrSAeDPJXXzv3
		      8pFTlcpEWpN2u52mabfb3rptS7PZarXjuCJ5NlB4Y6J+rxj0KM/ltZPHT5x8bWWl12y201ozMokxUXdsqtlqGsutVrPZqhtLClGl
		      sKyWTRBsjHfCMI16g41AsbLaEy/EHJDBBLAOmzmpKkiJwTB5Jv3Mn3nr7JFjR20S1dO0Wa/X0qQ73tq6bWOrlab1KpHz4ojBxmQD
		      VxQy6MuxY6+98uqRbOCazVa71WWOI5tOTU2naSOObatVT9MaG4BUvBCZ0LNtyCeUvUNaq0Po0KEXj75y+L779h144AHvfW+1R0yq
		      SsysI0lUNm2RILFcUfSi2O9/YG8cVatxpZpU4sQ2mnVmZVaQOgdChSDeDYo8n5ubefHFl86cPpPn0mi0O+3xKKpENtmwYVOj2agk
		      UbPZSNMUUIWoCg0Pl4jo8sUZETIcJ6by+utvfO9fnjj47DNXb14a5Kuf/NTHv/61r+/YsWN2Zs6YiNiU2F43LojAexWRPM8JDBCR
		      UScM8t6rivfOez/I+tC4yCjP88XFxVOnTh0+fPjq1atMtl7vtFqtOK7W08b09IZarVGrJZ2xZhRFzhVBvo9Kq5Tnly7cZI7ho2Mv
		      n/j37//niy8cWliY8dzLfW95dWZyYuofvvnNhx/+3NzsEpVcR6rqvR9NOWXS1TkngDiXSyGGIhEREVd458Q5r2IXFwYXL1w4euzY
		      qVOnV1aWk6TSbnfSWjtJku5Yd2ysW6836vVmrZYYKwp450WGYgKkI1BfungjtvUL527+7OlnWs1Of3Xpxz998pkXn44SVeRZNjDG
		      fPfxf/3Mw4/MzcwZDkp+TaWIlE1UVcKXcx5eICpCee4IkXd69eq18+fPnzp15s0zby0trYpoJal2Ot1qNU3T+uTkRLfbrdcbtVo1
		      jmKB8z5ThXgVKftDmNnKQhQRgFdXBtZUvedKJf3y7//BxObG93/wGMjZ2LjCffWPv/rUk7ffsfM9g0E/yGljzDDTVDZ7MJNRBYyI
		      5M4XQRJdunTt+PETb54+e+36xdX+gniK4loj7aRpK601O53Oho3dZrPZaDRqaVVVnSvEOTYocicKEEOhxEQ66hKWLSl8o1kzlglK
		      0GyQPfSxR/or+f8+/YQUEtloeXn+G3/z5//2vR/0+6tcFiUhsEeJOWUmAosKvDBFTObqlWuHDr3wyisn52YXXeEK56yt1+tpu9Ot
		      VWutVnvjxs3j4+20HtVqNYDCDKvqmEmkFPeqqhACDbsyIQht5/Px8bG0nuT93Gu+NLc0wds+//CX5+ZnD774QxaOoujHP3nyJz99
		      6rc//ZnFxQVmVhUNQpCVIBAwW+cKVVJobzU7dPClZ5555vr1GwTu9wfMplHvtFvTjUatM9aenOxu3Dhdb6SVimXjVZHnnlhZQQSR
		      oHAYUO+H43GpHxSAtWxUUG8k4+PNdy5dW1qef+yfvvXwp79w3/37vvjIF986e/LqzYtxxAAeffTRj//WJ4iYiELrVFVA2JAC3osK
		      q/K5sxef/J8fvXr8FedVPJzTetrsdsfb7Va3O7Zp86apqclWq5EkMTF5XzgnqmBmEUE5gA6Fw7Apa6kwKaSeGWQZ4ou7997Z7taU
		      3aWb5996+3VjpFarP/K5LxFiVWtN9OyzBw8f/nWtVvPe09qCYSImFYjwC88f+fu/+8fnnz88yPIid8x2cnJqx47b7rxz1/3vf99H
		      Hnz/fe/bvXX7dKtTiyvExgM+jLFByZTz57vHvaCMVVVAMIbNn/7JnxkyCq030h23b6tU49tv2/71r/1FkiRn3z6/ffvOCxcvXrl2
		      IYoS53utZvehhx7q9/uBNYNGVSHn1Av/4ufPPv6dJ25cnyNQNsg7nc7mTZt37rxt371777ln73vu2NbppnFsVZ33DpBgEBHxiP5F
		      guWgIr7UmMN+TcQgYSIiWINQmFoUPRDvu//uDx14X94bbN6yof5ak6nysQc//cqJwwAB8cGDz83NzQXqCKYU1BaFh0aHDh76znce
		      X1xYYY5coRs3btm8eeOeu3ft3n3Hho1TSWydz0UcCMxgJu8VYCb1KNl33QQ9dGgIxgQjQFTBYWgCMURJleCYnSLP8t7iymJ/sFyt
		      Rlu3bFte6r9n5+7pie1ZPqgmlZmZG3Nzc1EUAxQcKe/YmvTo0dce++dvLywsEGEwWJ6e3njHzt0ffuDD+/d/YPuODUnFKy+Tydlw
		      8CSMsUSGOWKO6P9dI6doOAMqIAymIJWJOY4jw2QM2EJJJyYnisKl1ea+vfcDNq12KnGjKByzMgsgqhLZ6NybF7/16OM3rixYiuFp
		      66bbdu/Z9cED9+zdd1en22BLZMBh7qbSwQm+FCDEOqT5AOtSYqBUBEO9wWsMCMCqYSlRAxVlkKo6FWHtDXo2YhHs3XP/Sy+8VIlr
		      W7duHxvrOl+IOiKjiqtXLj/2rW/PXe9NjW1jk7fajd133btn32137d3SbtVUVYRVYu8t1AGeCEJCrMzqnAtSESSAMpd+JaAqYSvD
		      Yi/FMAdtb9d62tpcSHGULMwvnTzxujVJNigmJqY2btqcF8uf/NRHx8e7S0sLqsZwxMQzN8/v2rXr3n0HiFSwPDU9Nj21ZXJDN0l5
		      aKYNn0tMJEpgYiUZAYAZIjoCMYGIiQ1Egnwv7ZRwRIFwh7ZY6RwES1eiKPYuFyFrEyJO4moSV+/avfkrX/3DlZUVa+JKpd5bzS9d
		      uAy1u3ft9j6amh7buLkeJaKenKonIZAxxnsfaqs0Smmt1AJoR/kcVSETKZOq1zVP4JbL7EikDh3HssJarUajUZ+92W81W7NzK/Pz
		      c+mNvHB5tdoc9Nx//9ePfvqTX5w/f0GKouhrqzX1oQP3/u7nH9y0uUtgoiiIsmGmmSh4FGAiD4RyLIoizJnilUAyup4J3ocRCSPL
		      aN2y602w4NQQkfNZpdqamOjMz64MstWnnv7h4tLswskLFy9cqdcGf/WXf3vs6KtEFEcxkzdUGQxWjeEkSeIk8YX362bldUllX7rF
		      OqzIsjcTl4bnKN8gAYQYBCEodM0xIyK73jYvnQ4VY+Al237b5vPnr9y8ee3Nt94ofH/rps1HXn71ie/+xzuXbzabTTYQKWJj4KnR
		      aNx1566xsXFSY00sTphKvyfcn5mhpcPNDFV4540xzgndstZtkoMYCxC6ZZPMYQsKAhjECiZigng3Ndn+yAPvr6eVqcmxTrOpot/4
		      2l/fuHat2UihKk6siRRkrPnQ/g/cs++eWq3ug4dnQr2XJxuM1nV0hhHxrRXfLUGP4Hvrhoa/Wxu6KBQKlCoKEKOemCgb9C9fOFev
		      1H01W1m6XuTLKy6vVaPYNggRw2a+uP/DBx767CfGpxueclgt0FMGldYZwplqkA+kIl4ERGQjFvXMcE4QSFDhvTcGQZeG+Qhr71XW
		      fH4e4WLoK0IVCiH2xMIse/bsufvu9y4urhY5F46c1ziugpC7zGn+yU89+Edf+fLOO7YoFQqHstiI1lhiKHrCMM1smEe2Scj/sPkN
		      xZzhd6X2Xcixt+J5ZCfAq8uL3m23b7l08Z39+/e/886Vn//qZ2na6Ha7/UHeaCS79uz+7O98Zv8D91aqUZYt2sgOJc4aeYFI1ts+
		      zMwqKA1O5jDfS7gybAeg4DSvFyTrq7pkj99cIsKGFd75wQf33/vLX7zw4Ecf/PXRX+/ec9eBAwc2bJh879737tx5e72ervZnC98z
		      1ogURAYKaHjv5kMxqSI4rBSiNFD1pXqGMjMxkZJhFvLM7L2MIGGMCdNxgEe4gzGGFm8sjfwAKl0lBVyJcDHQKM9w6LnDr7956ku/
		      94UNG6aJ1TnpD3reOyZHJEEqhImDlJXCkKtQFoH3XoRYleG8V+d8cB2c8yqUZWEO0DwrRMJ/g0aV0cwf4iaYElG3Bk0Yvj1TCHH5
		      Moo0VkSw4tUXzosXa41znqh8b0clMDTMcsM/rwtayagSnHfqnHeuzKB4FIUvCieirvDei/PeOwlyNBTiqOZC5/kNeOhIAxoiUhVm
		      r+oVBQAnhZIPWxVxNjIqKs6gFC+iENJS8g7hSEFBhrsSmEbOMpUDW0ieqjCziDIxDBFxmI9CskuuHkL9/wDvgVsCw0BSCAAAAABJ
		      RU5ErkJggg==";
		header('Content-type: image/png');
	}
	
	if ($picture==3) {
		$b64="iVBORw0KGgoAAAANSUhEUgAAAAcAAAABBAMAAADgJan9AAAAAXNSR0IArs4c6QAAABhQTFRF/////f398/Pz4ODg6+vr+fn51NTU
		      8PDwtw5ULwAAAAlwSFlzAAALEwAACxMBAJqcGAAAAAd0SU1FB9gDDBAJD2KU9CIAAAANSURBVAjXYxBVMWMAAAExAHDYpKh5AAAA
		      AElFTkSuQmCC";
		header('Content-type: image/png');
	}
	
	if ($picture==4) {
		$b64="iVBORw0KGgoAAAANSUhEUgAAAAcAAAABBAMAAADgJan9AAAAAXNSR0IArs4c6QAAABhQTFRF/////f398/Pz4ODg6+vr+fn51NTU
		      8PDwtw5ULwAAAAlwSFlzAAALEwAACxMBAJqcGAAAAAd0SU1FB9gDDBAINb2DHNEAAAANSURBVAjXY2AzURUAAAETAHBlDgLrAAAA
		      AElFTkSuQmCC";
		header('Content-type: image/png');
	}
	
	if ($picture==5) {
		$b64="R0lGODlhEgAKAIQSABAREhkaGzAzNDEzNTI0NklMT0tOUU9TVlFVWFJWWWVqbmxxdn2Dh3+FioGHjIKIjYmQlYuSl///////////
		      /////////////////////////////////////////////yH5BAEKAB8ALAAAAAASAAoAAAUo4CeOZGmOzKl+BrGaBRQQdC0ocLQe
		      ZtLsJ4QDUKsNFiqE62V6MJ+mEAA7";
		header('Content-type: image/gif');
	}
	
	if ($picture==6) {
		$b64="R0lGODlhEgAKAIQSABAREhkaGzAzNDEzNTI0NklMT0tOUU9TVlFVWFJWWWVqbmxxdn2Dh3+FioGHjIKIjYmQlYuSl///////////
		      /////////////////////////////////////////////yH5BAEKAB8ALAAAAAASAAoAAAUq4CeOZGmeH4OuhIEqAiHPAVScxxrd
		      ZY42CdNiMJsBHIiViZBUlh7O6CcEADs=";
		header('Content-type: image/gif');
	}
	
	

	echo base64_decode($b64);
}

// ***PIC-end

function nice_chars($str){
	return  str_replace("_"," ",$str);
}

function nice_filename($filename){
	$nice_filename=basename($filename);
	/*if (substr($nice_filename,0,2)=="./"){
		$nice_filename=substr($nice_filename,2);
	}*/
	
	$nice_filename=nice_chars($nice_filename);
	return $nice_filename;
}

function my_htmlentities($str){
	return htmlentities($str,null,"UTF-8");
}

function array_isearch($str, $array) {
	foreach($array as $k => $v) {
  	if(strcasecmp($str, $v) == 0) return $k;
  }
  return false;
}

function my_glob($path,$exts=null,$dirs=false,$dotdot=false){
	$files = array();
	if(is_dir($path)){
		if( ($dh=opendir($path)) ) {
			while(($file = readdir($dh)) !== false){
				if ($file==".") continue;
				if ($file==".." && !$dotdot) continue;
				
				if (!is_null($exts)){
					$pi=pathinfo($file);
					$ext=$pi['extension'];
					if (is_array($exts)){
						if (array_isearch($ext,$exts)===false) continue;
					}else{
						if(strcasecmp($ext, $exts) !== 0) continue;
					}
				}
				
				$ak=$path.DIRECTORY_SEPARATOR.$file;
				if (is_dir($ak)==$dirs) $files[]=$ak;
			}
		}
	}
	//print_r($files);
	return $files;
}

?>